package ch.epfl.tpp;

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.NoSuchElementException;

public class InputStreamToIterable implements Iterable<Byte> {
    private InputStream inStream;

    public InputStreamToIterable(InputStream inStream) {
        this.inStream = inStream;
    }

    @Override
    public Iterator<Byte> iterator() {
        return new Iterator<Byte>() {
            /* Invariant: next is valid iff hasNext is true. */
            byte next;
            boolean hasNext;

            { advanceToNext(); }

            @Override
            public boolean hasNext() {
                return hasNext;
            }

            @Override
            public Byte next() {
                if (!hasNext())
                    throw new NoSuchElementException();
                byte savedNext = next;
                advanceToNext();
                return savedNext;
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }

            private void advanceToNext() {
                try {
                    int nextAsInt = inStream.read();
                    hasNext = (nextAsInt != -1);
                    next = (byte)nextAsInt;
                    if (nextAsInt == -1)
                        inStream.close();
                } catch (IOException e) {
                    hasNext = false;
                }
            }
        };
    }
}
